'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');
var rxjs = require('rxjs');

var areInputsEqual = function (newInputs, oldInputs) {
    if (newInputs.length !== oldInputs.length) {
        return false;
    }
    for (var i = 0; i < newInputs.length; i++) {
        if (newInputs[i] !== oldInputs[i]) {
            return false;
        }
    }
    return true;
};
function memoizeOne(resultFn) {
    var lastThis;
    var lastArgs = [];
    var lastResult;
    var calledOnce = false;
    // breaking cache when context (this) or arguments change
    function memoized() {
        var newArgs = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            newArgs[_i] = arguments[_i];
        }
        if (calledOnce && lastThis === this && areInputsEqual(newArgs, lastArgs)) {
            return lastResult;
        }
        // Throwing during an assignment aborts the assignment: https://codepen.io/alexreardon/pen/RYKoaz
        // Doing the lastResult assignment first so that if it throws
        // nothing will be overwritten
        lastResult = resultFn.apply(this, newArgs);
        calledOnce = true;
        lastThis = this;
        lastArgs = newArgs;
        return lastResult;
    }
    return memoized;
}

var getLastSyncEmission = function (source$) {
    var value;
    var subscription = source$.subscribe(function (v) { return value = v; });
    subscription.unsubscribe();
    return value;
};

function survivableShare(survivalCount) {
    var subject;
    var refCount = 0;
    var deathCount = 0;
    var subscription;
    return function (source$) { return new rxjs.Observable(function (observer) {
        refCount++;
        var innerSubscription;
        if (!subscription) {
            subject = new rxjs.Subject();
            innerSubscription = subject.subscribe(observer);
            subscription = source$.subscribe({
                next: function (value) { subject.next(value); },
                error: function (err) { subject.error(err); },
                complete: function () { subject.complete(); }
            });
        }
        else {
            innerSubscription = subject.subscribe(observer);
        }
        return function teardown() {
            refCount--;
            innerSubscription.unsubscribe();
            if (refCount === 0) {
                deathCount++;
                if (deathCount > survivalCount) {
                    subscription.unsubscribe();
                    deathCount = 0;
                    subscription = undefined;
                    subject = undefined;
                }
            }
        };
    }); };
}

var resolveInitialValue = function (source$, operatorFunction, initialValue) {
    var enhancedSource$ = operatorFunction(source$);
    var lastSyncEmission = getLastSyncEmission(enhancedSource$);
    return lastSyncEmission === undefined ? initialValue : lastSyncEmission;
};
var useObservable = function (source$, initialValue) {
    var operatorFunction = react.useState(function () { return memoizeOne(survivableShare(1)); })[0];
    var _a = react.useState(function () { return resolveInitialValue(source$, operatorFunction, initialValue); }), emittedValue = _a[0], setEmittedValue = _a[1];
    react.useEffect(function () {
        var enhancedSource$ = operatorFunction(source$);
        var subscription = enhancedSource$.subscribe(setEmittedValue);
        return function () { return subscription.unsubscribe(); };
    }, [source$, setEmittedValue, operatorFunction]);
    return emittedValue;
};

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var resolveInitialValue$1 = function (source$, initialValue) {
    var lastSyncEmission = getLastSyncEmission(source$);
    return typeof lastSyncEmission === 'undefined' ? initialValue : lastSyncEmission;
};
var withObservable = function (propName, source$, initialValue) { return function (BaseComponent) {
    var factory = react.createFactory(BaseComponent);
    var enhancedSource$ = source$.pipe(survivableShare(1));
    return /** @class */ (function (_super) {
        __extends(class_1, _super);
        function class_1(props) {
            var _this = _super.call(this, props) || this;
            _this.state = { emittedValue: resolveInitialValue$1(enhancedSource$, initialValue) };
            return _this;
        }
        class_1.prototype.setObservableValue = function (val) {
            this.setState({ emittedValue: val });
        };
        class_1.prototype.componentDidMount = function () {
            this.subscription = enhancedSource$
                .subscribe(this.setObservableValue);
        };
        class_1.prototype.componentWillUnmount = function () {
            var _a;
            (_a = this.subscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
        };
        class_1.prototype.render = function () {
            var _a;
            return factory(__assign(__assign({}, this.props), (_a = {}, _a[propName] = this.state.emittedValue, _a)));
        };
        return class_1;
    }(react.Component));
}; };

exports.useObservable = useObservable;
exports.withObservable = withObservable;
